@@ -58,9 +58,7 @@ namespace util
 	bool Daemon_Singleton::IsService () const
 	{
 		bool service = false;
-#ifndef _WIN32
 		i2p::config::GetOption("service", service);
-#endif
 		return service;
 	}
 
@@ -155,6 +153,9 @@ namespace util
 		bool aesni; i2p::config::GetOption("cpuext.aesni", aesni);
 		bool avx; i2p::config::GetOption("cpuext.avx", avx);
 		bool forceCpuExt; i2p::config::GetOption("cpuext.force", forceCpuExt);
+		bool ssu; i2p::config::GetOption("ssu", ssu);
+		if (!ssu && i2p::config::IsDefault ("precomputation.elgamal"))
+			precomputation = false; // we don't elgamal table if no ssu, unless it's specified explicitly
 		i2p::crypto::InitCrypto (precomputation, aesni, avx, forceCpuExt);
 
 		int netID; i2p::config::GetOption("netid", netID);
@@ -252,17 +253,17 @@ namespace util
 			if (!ipv4 && !ipv6)
 				i2p::context.SetStatus (eRouterStatusMesh);
 		}
+		if (!ssu) i2p::context.RemoveSSUAddress (); // TODO: remove later
 		bool ssu2; i2p::config::GetOption("ssu2.enabled", ssu2);
 		if (ssu2)
 		{
+			uint16_t ssu2port; i2p::config::GetOption("ssu2.port", ssu2port);
+			if (!ssu2port) ssu2port = ssu ? (port + 1) : port;
 			bool published; i2p::config::GetOption("ssu2.published", published);
 			if (published)
-			{
-				uint16_t ssu2port; i2p::config::GetOption("ssu2.port", ssu2port);
 				i2p::context.PublishSSU2Address (ssu2port, true, ipv4, ipv6); // publish
-			}
 			else
-				i2p::context.PublishSSU2Address (0, false, ipv4, ipv6); // unpublish
+				i2p::context.PublishSSU2Address (ssu2port, false, ipv4, ipv6); // unpublish
 		}
 
 		bool transit; i2p::config::GetOption("notransit", transit);
@@ -406,7 +407,7 @@ namespace util
 
 		i2p::transport::transports.SetCheckReserved(checkInReserved);
 		i2p::transport::transports.Start(ntcp2, ssu, ssu2);
-		if (i2p::transport::transports.IsBoundSSU() || i2p::transport::transports.IsBoundNTCP2())
+		if (i2p::transport::transports.IsBoundSSU() || i2p::transport::transports.IsBoundSSU2() || i2p::transport::transports.IsBoundNTCP2())
 			LogPrint(eLogInfo, "Daemon: Transports started");
 		else
 		{
