@@ -386,7 +386,10 @@ namespace data
 								((it.iHost.is_v4 () && address->IsV4 ()) || (it.iHost.is_v6 () && address->IsV6 ())))
 								numValid++;
 							else
+							{	
 								it.iPort = 0;
+								if (isV2) numValid++;
+							}	
 						}
 						if (numValid)
 							m_ReachableTransports |= supportedTransports;
@@ -409,28 +412,53 @@ namespace data
 					if (address->host.is_v4 ()) m_ReachableTransports |= eSSU2V4;
 					if (address->host.is_v6 ()) m_ReachableTransports |= eSSU2V6;
 				}
+				if (address->transportStyle == eTransportSSU2)
+				{
+					if (address->port) address->published = true;
+					if (address->ssu && !address->ssu->introducers.empty ())
+					{
+						// exclude invalid introducers
+						uint32_t ts = i2p::util::GetSecondsSinceEpoch ();
+						int numValid = 0;
+						for (auto& it: address->ssu->introducers)
+						{
+							if (it.iTag && ts <= it.iExp)						
+								numValid++;
+							else	
+								it.iTag = 0;
+						}
+						if (numValid)
+							m_ReachableTransports |= supportedTransports;
+						else
+							address->ssu->introducers.resize (0);
+					}	
+				}
+				else
+				{
+					// create additional SSU2 address. TODO: remove later
+					auto ssu2addr = std::make_shared<Address> ();
+					ssu2addr->transportStyle = eTransportSSU2;
+					ssu2addr->host = address->host; ssu2addr->port = address->port;
+					ssu2addr->s = address->s; ssu2addr->i = iV2;
+					ssu2addr->date = address->date; ssu2addr->caps = address->caps;
+					ssu2addr->published = address->published;
+					ssu2addr->ssu.reset (new SSUExt ()); ssu2addr->ssu->mtu = address->ssu->mtu;
+					uint32_t ts = i2p::util::GetSecondsSinceEpoch ();
+					if (!address->ssu->introducers.empty ())
+					{	
+						for (const auto& introducer: address->ssu->introducers)
+							if (!introducer.iPort && introducer.iHost.is_unspecified () && ts < introducer.iExp) // SSU2
+								ssu2addr->ssu->introducers.push_back (introducer);
+						if (!ssu2addr->ssu->introducers.empty ())
+							m_ReachableTransports |= supportedTransports;
+					}	
+					addresses->push_back(ssu2addr);
+				}	
 			}
 			if (supportedTransports)
 			{
 				if (!(m_SupportedTransports & supportedTransports)) // avoid duplicates
-				{
 					addresses->push_back(address);
-					if (address->transportStyle == eTransportSSU && isV2)
-					{
-						// create additional SSU2 address. TODO: remove later
-						auto ssu2addr = std::make_shared<Address> ();
-						ssu2addr->transportStyle = eTransportSSU2;
-						ssu2addr->host = address->host; ssu2addr->port = address->port;
-						ssu2addr->s = address->s; ssu2addr->i = iV2;
-						ssu2addr->date = address->date; ssu2addr->caps = address->caps;
-						ssu2addr->published = address->published;
-						ssu2addr->ssu.reset (new SSUExt ()); ssu2addr->ssu->mtu = address->ssu->mtu;
-						for (const auto& introducer: address->ssu->introducers)
-							if (!introducer.iPort) // SSU2
-								ssu2addr->ssu->introducers.push_back (introducer);
-						addresses->push_back(ssu2addr);
-					}
-				}
 				m_SupportedTransports |= supportedTransports;
 			}
 		}
@@ -680,6 +708,7 @@ namespace data
 	{
 		auto addr = std::make_shared<Address>();
 		addr->transportStyle = eTransportSSU2;
+		addr->port = 0;
 		addr->caps = caps;
 		addr->date = 0;
 		addr->ssu.reset (new SSUExt ());
@@ -699,7 +728,7 @@ namespace data
 		addr->host = host;
 		addr->port = port;
 		addr->published = true;
-		addr->caps = 0;
+		addr->caps = i2p::data::RouterInfo::eSSUTesting | i2p::data::RouterInfo::eSSUIntroducer; // BC;
 		addr->date = 0;
 		addr->ssu.reset (new SSUExt ());
 		addr->ssu->mtu = 0;
@@ -905,6 +934,21 @@ namespace data
 			});
 	}
 
+	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetSSU2Address (bool v4) const
+	{
+		if (v4)
+		{	
+			if (m_SupportedTransports & eSSU2V4)
+				return GetSSU2V4Address ();
+		}
+		else
+		{
+			if (m_SupportedTransports & eSSU2V6)
+				return GetSSU2V6Address ();
+		}	
+		return nullptr;
+	}	
+		
 	template<typename Filter>
 	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetAddress (Filter filter) const
 	{
@@ -936,7 +980,8 @@ namespace data
 		return GetAddress (
 			[key, isV6](std::shared_ptr<const RouterInfo::Address> address)->bool
 			{
-				return address->IsSSU2 () && !memcmp (address->s, key, 32) && address->IsV6 () == isV6;
+				return address->IsSSU2 () && !memcmp (address->s, key, 32) && 
+					((isV6 && address->IsV6 ()) || (!isV6 && address->IsV4 ()));
 			});
 	}
 
@@ -1000,6 +1045,17 @@ namespace data
 			});
 	}
 
+	bool RouterInfo::IsSSU2PeerTesting (bool v4) const
+	{
+		if (!(m_SupportedTransports & (v4 ? eSSU2V4 : eSSU2V6))) return false;
+		return (bool)GetAddress (
+			[v4](std::shared_ptr<const RouterInfo::Address> address)->bool
+			{
+				return (address->IsSSU2 ()) && address->IsPeerTesting () &&
+					((v4 && address->IsV4 ()) || (!v4 && address->IsV6 ())) && address->IsReachableSSU ();
+			});
+	}	
+		
 	bool RouterInfo::IsIntroducer (bool v4) const
 	{
 		if (!(m_SupportedTransports & (v4 ? eSSUV4 : eSSUV6))) return false;
@@ -1011,6 +1067,17 @@ namespace data
 			});
 	}
 
+	bool RouterInfo::IsSSU2Introducer (bool v4) const
+	{
+		if (!(m_SupportedTransports & (v4 ? eSSU2V4 : eSSU2V6))) return false;
+		return (bool)GetAddress (
+			[v4](std::shared_ptr<const RouterInfo::Address> address)->bool
+			{
+				return (address->IsSSU2 ()) && address->IsIntroducer () &&
+					((v4 && address->IsV4 ()) || (!v4 && address->IsV6 ())) && !address->host.is_unspecified ();
+			});
+	}
+		
 	void RouterInfo::SetUnreachableAddressesTransportCaps (uint8_t transports)
 	{
 		for (auto& addr: *m_Addresses)
@@ -1031,21 +1098,29 @@ namespace data
 		for (const auto& addr: *m_Addresses)
 		{
 			uint8_t transports = 0;
-			if (addr->transportStyle == eTransportNTCP)
-			{
-				if (addr->IsV4 ()) transports |= eNTCP2V4;
-				if (addr->IsV6 ())
-					transports |= (i2p::util::net::IsYggdrasilAddress (addr->host) ? eNTCP2V6Mesh : eNTCP2V6);
-				if (addr->IsPublishedNTCP2 ())
-					m_ReachableTransports |= transports;
-			}
-			else if (addr->transportStyle == eTransportSSU)
-			{
-				if (addr->IsV4 ()) transports |= eSSUV4;
-				if (addr->IsV6 ()) transports |= eSSUV6;
-				if (addr->IsReachableSSU ())
-					m_ReachableTransports |= transports;
-			}
+			switch (addr->transportStyle)
+			{        
+				case eTransportNTCP:
+					if (addr->IsV4 ()) transports |= eNTCP2V4;
+					if (addr->IsV6 ())
+						transports |= (i2p::util::net::IsYggdrasilAddress (addr->host) ? eNTCP2V6Mesh : eNTCP2V6);
+					if (addr->IsPublishedNTCP2 ())
+						m_ReachableTransports |= transports;
+				break;
+				case eTransportSSU:
+					if (addr->IsV4 ()) transports |= eSSUV4;
+					if (addr->IsV6 ()) transports |= eSSUV6;
+					if (addr->IsReachableSSU ())
+						m_ReachableTransports |= transports;
+				break;
+				case eTransportSSU2:
+					if (addr->IsV4 ()) transports |= eSSU2V4;
+					if (addr->IsV6 ()) transports |= eSSU2V6;
+					if (addr->IsReachableSSU ())
+						m_ReachableTransports |= transports;
+				break;	
+				default: ;	
+			}	
 			m_SupportedTransports |= transports;
 		}
 	}
@@ -1214,6 +1289,7 @@ namespace data
 				if (address.published)
 				{
 					isPublished = true;
+					if (address.IsPeerTesting ()) caps += CAPS_FLAG_SSU_TESTING;
 					if (address.IsIntroducer ()) caps += CAPS_FLAG_SSU_INTRODUCER;
 				}
 				else
@@ -1233,7 +1309,7 @@ namespace data
 			else
 				WriteString ("", s);
 
-			if (isPublished)
+			if (isPublished && !address.host.is_unspecified ())
 			{
 				WriteString ("host", properties);
 				properties << '=';
@@ -1336,7 +1412,7 @@ namespace data
 					properties << ';';
 				}
 			}
-			if (isPublished || (address.ssu && !address.IsSSU2 ()))
+			if ((isPublished || (address.ssu && !address.IsSSU2 ())) && address.port)
 			{
 				WriteString ("port", properties);
 				properties << '=';
@@ -1404,5 +1480,40 @@ namespace data
 	{
 		return std::make_shared<Buffer> ();
 	}
+
+	bool LocalRouterInfo::AddSSU2Introducer (const Introducer& introducer, bool v4)
+	{
+		for (auto& addr : GetAddresses ())
+		{
+			if (addr->IsSSU2 () && ((v4 && addr->IsV4 ()) || (!v4 && addr->IsV6 ())))
+			{
+				for (auto& intro: addr->ssu->introducers)
+					if (intro.iTag == introducer.iTag) return false; // already presented
+				addr->ssu->introducers.push_back (introducer);
+				SetReachableTransports (GetReachableTransports () | ((addr->IsV4 () ? eSSU2V4 : eSSU2V6)));
+				return true;
+			}
+		}
+		return false;
+	}	
+
+	bool LocalRouterInfo::RemoveSSU2Introducer (const IdentHash& h, bool v4)
+	{
+		for (auto& addr: GetAddresses ())
+		{
+			if (addr->IsSSU2 () && ((v4 && addr->IsV4 ()) || (!v4 && addr->IsV6 ())))
+			{
+				for (auto it = addr->ssu->introducers.begin (); it != addr->ssu->introducers.end (); ++it)
+					if (h == it->iKey)
+					{
+						addr->ssu->introducers.erase (it);
+						if (addr->ssu->introducers.empty ())
+							SetReachableTransports (GetReachableTransports () & ~(addr->IsV4 () ? eSSU2V4 : eSSU2V6));
+						return true;
+					}
+			}
+		}
+		return false;
+	}	
 }
 }
