@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2021, The PurpleI2P Project
+* Copyright (c) 2013-2022, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -19,21 +19,14 @@
 #include "I2NPProtocol.h"
 #include "Identity.h"
 #include "RouterInfo.h"
+#include "TransportSession.h"
 
 namespace i2p
 {
 namespace transport
 {
-
 	const size_t SSU_MTU_V4 = 1484;
-	#ifdef MESHNET
-	const size_t SSU_MTU_V6 = 1286;
-	#else
 	const size_t SSU_MTU_V6 = 1488;
-	#endif
-	const size_t IPV4_HEADER_SIZE = 20;
-	const size_t IPV6_HEADER_SIZE = 40;
-	const size_t UDP_HEADER_SIZE = 8;
 	const size_t SSU_V4_MAX_PACKET_SIZE = SSU_MTU_V4 - IPV4_HEADER_SIZE - UDP_HEADER_SIZE; // 1456
 	const size_t SSU_V6_MAX_PACKET_SIZE = SSU_MTU_V6 - IPV6_HEADER_SIZE - UDP_HEADER_SIZE; // 1440
 	const int RESEND_INTERVAL = 3; // in seconds
@@ -79,7 +72,7 @@ namespace transport
 		uint64_t receivedFragmentsBits;
 		std::set<std::shared_ptr<Fragment>, FragmentCmp> savedFragments;
 
-		IncompleteMessage (std::shared_ptr<I2NPMessage> m): msg (m), nextFragmentNum (0),
+		IncompleteMessage (std::shared_ptr<I2NPMessage>&& m): msg (m), nextFragmentNum (0),
 			lastFragmentInsertTime (0), receivedFragmentsBits (0) {};
 		void AttachNextFragment (const uint8_t * fragment, size_t fragmentSize);
 	};
