@@ -233,6 +233,7 @@ namespace transport
 				{
 					if (m_NTCP2Server) m_NTCP2Server->SetLocalAddress (addr);
 					if (m_SSUServer) m_SSUServer->SetLocalAddress (addr);
+					if (m_SSU2Server) m_SSU2Server->SetLocalAddress (addr);
 				}
 			}
 		}
@@ -249,6 +250,7 @@ namespace transport
 				{
 					if (m_NTCP2Server) m_NTCP2Server->SetLocalAddress (addr);
 					if (m_SSUServer) m_SSUServer->SetLocalAddress (addr);
+					if (m_SSU2Server) m_SSU2Server->SetLocalAddress (addr);
 				}
 			}
 		}
@@ -268,6 +270,7 @@ namespace transport
 
 		// start servers
 		if (m_NTCP2Server) m_NTCP2Server->Start ();
+		if (m_SSU2Server) m_SSU2Server->Start ();
 		if (m_SSUServer)
 		{
 			LogPrint (eLogInfo, "Transports: Start listening UDP port ", ssuPort);
@@ -282,9 +285,8 @@ namespace transport
 				delete m_SSUServer;
 				m_SSUServer = nullptr;
 			}
-			if (m_SSUServer) DetectExternalIP ();
 		}
-		if (m_SSU2Server) m_SSU2Server->Start ();
+		if (m_SSUServer || m_SSU2Server) DetectExternalIP ();
 
 		m_PeerCleanupTimer->expires_from_now (boost::posix_time::seconds(5*SESSION_CREATION_TIMEOUT));
 		m_PeerCleanupTimer->async_wait (std::bind (&Transports::HandlePeerCleanupTimer, this, std::placeholders::_1));
@@ -413,9 +415,10 @@ namespace transport
 				auto r = netdb.FindRouter (ident);
 				if (r && (r->IsUnreachable () || !r->IsReachableFrom (i2p::context.GetRouterInfo ()))) return; // router found but non-reachable
 				{
+					auto ts = i2p::util::GetSecondsSinceEpoch ();
 					std::unique_lock<std::mutex> l(m_PeersMutex);
 					it = m_Peers.insert (std::pair<i2p::data::IdentHash, Peer>(ident, { 0, r, {},
-						i2p::util::GetSecondsSinceEpoch (), {} })).first;
+						ts, ts + PEER_ROUTER_INFO_UPDATE_INTERVAL, {} })).first;
 				}
 				connected = ConnectToPeer (ident, it->second);
 			}
@@ -450,7 +453,7 @@ namespace transport
 			peer.router = netdb.FindRouter (ident); // try to get new one from netdb
 		if (peer.router) // we have RI already
 		{
-			if (peer.numAttempts < 2) // NTCP2, 0 - ipv6, 1- ipv4
+			if (peer.numAttempts < 2) // NTCP2, 0 - ipv6, 1 - ipv4
 			{
 				if (m_NTCP2Server) // we support NTCP2
 				{
@@ -488,60 +491,12 @@ namespace transport
 				else
 					peer.numAttempts = 2; // switch to SSU
 			}
-			if (peer.numAttempts == 2 || peer.numAttempts == 3) // SSU
-			{
-				if (m_SSUServer)
-				{
-					std::shared_ptr<const RouterInfo::Address> address;
-					if (peer.numAttempts == 2) // SSU ipv6
-					{
-						if (context.GetRouterInfo ().IsSSUV6 () && peer.router->IsReachableBy (RouterInfo::eSSUV6))
-						{
-							address = peer.router->GetSSUV6Address ();
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (!address && peer.numAttempts == 3) // SSU ipv4
-					{
-						if (context.GetRouterInfo ().IsSSU (true) && peer.router->IsReachableBy (RouterInfo::eSSUV4))
-						{
-							address = peer.router->GetSSUAddress (true);
-							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
-								address = nullptr;
-						}
-						peer.numAttempts++;
-					}
-					if (address && address->IsReachableSSU ())
-					{
-						if (m_SSUServer->CreateSession (peer.router, address))
-							return true;
-					}
-				}
-				else
-					peer.numAttempts += 2; // switch to Mesh
-			}
-			if (peer.numAttempts == 4) // Mesh
-			{
-				peer.numAttempts++;
-				if (m_NTCP2Server && context.GetRouterInfo ().IsMesh () && peer.router->IsMesh ())
-				{
-					auto address = peer.router->GetYggdrasilAddress ();
-					if (address)
-					{
-						auto s = std::make_shared<NTCP2Session> (*m_NTCP2Server, peer.router, address);
-						m_NTCP2Server->Connect (s);
-						return true;
-					}
-				}
-			}
-			if (peer.numAttempts == 5 || peer.numAttempts == 6) // SSU2
+			if (peer.numAttempts == 2 || peer.numAttempts == 3) // SSU2, 2 - ipv6, 3 - ipv4
 			{
 				if (m_SSU2Server)
 				{
 					std::shared_ptr<const RouterInfo::Address> address;
-					if (peer.numAttempts == 5) // SSU2 ipv6
+					if (peer.numAttempts == 2) // SSU2 ipv6
 					{
 						if (context.GetRouterInfo ().IsSSU2V6 () && peer.router->IsReachableBy (RouterInfo::eSSU2V6))
 						{
@@ -551,7 +506,7 @@ namespace transport
 						}
 						peer.numAttempts++;
 					}
-					if (!address && peer.numAttempts == 6) // SSU2 ipv4
+					if (!address && peer.numAttempts == 3) // SSU2 ipv4
 					{
 						if (context.GetRouterInfo ().IsSSU2V4 () && peer.router->IsReachableBy (RouterInfo::eSSU2V4))
 						{
@@ -568,7 +523,55 @@ namespace transport
 					}
 				}
 				else
-					peer.numAttempts += 2;
+					peer.numAttempts += 2; // switch to mesh
+			}
+			if (peer.numAttempts == 4) // Mesh
+			{
+				peer.numAttempts++;
+				if (m_NTCP2Server && context.GetRouterInfo ().IsMesh () && peer.router->IsMesh ())
+				{
+					auto address = peer.router->GetYggdrasilAddress ();
+					if (address)
+					{
+						auto s = std::make_shared<NTCP2Session> (*m_NTCP2Server, peer.router, address);
+						m_NTCP2Server->Connect (s);
+						return true;
+					}
+				}
+			}
+			if (peer.numAttempts == 5 || peer.numAttempts == 6) // SSU, 5 - ipv6, 6 - ipv4
+			{
+				if (m_SSUServer)
+				{
+					std::shared_ptr<const RouterInfo::Address> address;
+					if (peer.numAttempts == 5) // SSU ipv6
+					{
+						if (context.GetRouterInfo ().IsSSUV6 () && peer.router->IsReachableBy (RouterInfo::eSSUV6))
+						{
+							address = peer.router->GetSSUV6Address ();
+							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
+								address = nullptr;
+						}
+						peer.numAttempts++;
+					}
+					if (!address && peer.numAttempts == 6) // SSU ipv4
+					{
+						if (context.GetRouterInfo ().IsSSU (true) && peer.router->IsReachableBy (RouterInfo::eSSUV4))
+						{
+							address = peer.router->GetSSUAddress (true);
+							if (address && m_CheckReserved && i2p::util::net::IsInReservedRange(address->host))
+								address = nullptr;
+						}
+						peer.numAttempts++;
+					}
+					if (address && address->IsReachableSSU ())
+					{
+						if (m_SSUServer->CreateSession (peer.router, address))
+							return true;
+					}
+				}
+				else
+					peer.numAttempts += 2; 
 			}
 			LogPrint (eLogInfo, "Transports: No compatble NTCP2 or SSU addresses available");
 			i2p::data::netdb.SetUnreachable (ident, true); // we are here because all connection attempts failed
@@ -619,68 +622,109 @@ namespace transport
 			i2p::context.SetStatus (eRouterStatusOK);
 			return;
 		}
-		if (m_SSUServer)
+		if (m_SSUServer || m_SSU2Server)
 			PeerTest ();
 		else
-			LogPrint (eLogError, "Transports: Can't detect external IP. SSU is not available");
+			LogPrint (eLogWarning, "Transports: Can't detect external IP. SSU or SSU2 is not available");
 	}
 
 	void Transports::PeerTest (bool ipv4, bool ipv6)
 	{
-		if (RoutesRestricted() || !m_SSUServer) return;
+		if (RoutesRestricted() || (!m_SSUServer && !m_SSU2Server)) return;
 		if (ipv4 && i2p::context.SupportsV4 ())
 		{
 			LogPrint (eLogInfo, "Transports: Started peer test IPv4");
 			std::set<i2p::data::IdentHash> excluded;
 			excluded.insert (i2p::context.GetIdentHash ()); // don't pick own router
-			bool statusChanged = false;
-			for (int i = 0; i < 5; i++)
+			if (m_SSUServer)
 			{
-				auto router = i2p::data::netdb.GetRandomPeerTestRouter (true, excluded); // v4
-				if (router)
+				bool statusChanged = false;
+				for (int i = 0; i < 5; i++)
 				{
-					auto addr = router->GetSSUAddress (true); // ipv4
-					if (addr && !i2p::util::net::IsInReservedRange(addr->host))
+					auto router = i2p::data::netdb.GetRandomPeerTestRouter (true, excluded); // v4
+					if (router)
 					{
-						if (!statusChanged)
+						auto addr = router->GetSSUAddress (true); // ipv4
+						if (addr && !i2p::util::net::IsInReservedRange(addr->host))
 						{
-							statusChanged = true;
-							i2p::context.SetStatus (eRouterStatusTesting); // first time only
+							if (!statusChanged)
+							{
+								statusChanged = true;
+								i2p::context.SetStatus (eRouterStatusTesting); // first time only
+							}
+							m_SSUServer->CreateSession (router, addr, true); // peer test v4
 						}
-						m_SSUServer->CreateSession (router, addr, true); // peer test v4
+						excluded.insert (router->GetIdentHash ());
+					}
+				}
+				if (!statusChanged)
+					LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv4");
+			}
+			// SSU2
+			if (m_SSU2Server)
+			{
+				excluded.clear ();
+				excluded.insert (i2p::context.GetIdentHash ());
+				for (int i = 0; i < 3; i++)
+				{
+					auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (true, excluded); // v4
+					if (router)
+					{
+						if (i2p::context.GetStatus () != eRouterStatusTesting)
+							i2p::context.SetStatusSSU2 (eRouterStatusTesting);
+						m_SSU2Server->StartPeerTest (router, true);
+						excluded.insert (router->GetIdentHash ());
 					}
-					excluded.insert (router->GetIdentHash ());
 				}
 			}
-			if (!statusChanged)
-				LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv4");
 		}
 		if (ipv6 && i2p::context.SupportsV6 ())
 		{
 			LogPrint (eLogInfo, "Transports: Started peer test IPv6");
 			std::set<i2p::data::IdentHash> excluded;
 			excluded.insert (i2p::context.GetIdentHash ()); // don't pick own router
-			bool statusChanged = false;
-			for (int i = 0; i < 5; i++)
+			if (m_SSUServer)
 			{
-				auto router = i2p::data::netdb.GetRandomPeerTestRouter (false, excluded); // v6
-				if (router)
+				bool statusChanged = false;
+				for (int i = 0; i < 5; i++)
 				{
-					auto addr = router->GetSSUV6Address ();
-					if (addr && !i2p::util::net::IsInReservedRange(addr->host))
+					auto router = i2p::data::netdb.GetRandomPeerTestRouter (false, excluded); // v6
+					if (router)
 					{
-						if (!statusChanged)
+						auto addr = router->GetSSUV6Address ();
+						if (addr && !i2p::util::net::IsInReservedRange(addr->host))
 						{
-							statusChanged = true;
-							i2p::context.SetStatusV6 (eRouterStatusTesting); // first time only
+							if (!statusChanged)
+							{
+								statusChanged = true;
+								i2p::context.SetStatusV6 (eRouterStatusTesting); // first time only
+							}
+							m_SSUServer->CreateSession (router, addr, true); // peer test v6
 						}
-						m_SSUServer->CreateSession (router, addr, true); // peer test v6
+						excluded.insert (router->GetIdentHash ());
+					}
+				}
+				if (!statusChanged)
+					LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv6");
+			}
+
+			// SSU2
+			if (m_SSU2Server)
+			{
+				excluded.clear ();
+				excluded.insert (i2p::context.GetIdentHash ());
+				for (int i = 0; i < 3; i++)
+				{
+					auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (false, excluded); // v6
+					if (router)
+					{
+						if (i2p::context.GetStatusV6 () != eRouterStatusTesting)
+							i2p::context.SetStatusV6SSU2 (eRouterStatusTesting);
+						m_SSU2Server->StartPeerTest (router, false);
+						excluded.insert (router->GetIdentHash ());
 					}
-					excluded.insert (router->GetIdentHash ());
 				}
 			}
-			if (!statusChanged)
-				LogPrint (eLogWarning, "Transports: Can't find routers for peer test IPv6");
 		}
 	}
 
@@ -731,8 +775,10 @@ namespace transport
 					return;
 				}
 				session->SendI2NPMessages ({ CreateDatabaseStoreMsg () }); // send DatabaseStore
-				std::unique_lock<std::mutex>	l(m_PeersMutex);
-				m_Peers.insert (std::make_pair (ident, Peer{ 0, nullptr, { session }, i2p::util::GetSecondsSinceEpoch (), {} }));
+				auto ts = i2p::util::GetSecondsSinceEpoch ();
+				std::unique_lock<std::mutex> l(m_PeersMutex);
+				m_Peers.insert (std::make_pair (ident, Peer{ 0, nullptr, { session },
+					ts, ts + PEER_ROUTER_INFO_UPDATE_INTERVAL, {} }));
 			}
 		});
 	}
@@ -793,7 +839,17 @@ namespace transport
 					it = m_Peers.erase (it);
 				}
 				else
+				{
+					if (ts > it->second.nextRouterInfoUpdateTime)
+					{
+						auto session = it->second.sessions.front ();
+						if (session)
+							session->SendLocalRouterInfo (true);
+						it->second.nextRouterInfoUpdateTime = ts + PEER_ROUTER_INFO_UPDATE_INTERVAL + 
+							rand () % PEER_ROUTER_INFO_UPDATE_INTERVAL_VARIANCE;
+					}
 					++it;
+				}
 			}
 			UpdateBandwidth (); // TODO: use separate timer(s) for it
 			bool ipv4Testing = i2p::context.GetStatus () == eRouterStatusTesting;
