@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2021, The PurpleI2P Project
+* Copyright (c) 2013-2022, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -7,6 +7,7 @@
 */
 
 #include <cassert>
+#include <boost/algorithm/string.hpp>
 #include "Base.h"
 #include "Log.h"
 #include "Destination.h"
@@ -340,7 +341,7 @@ namespace client
 			m_InHeader.clear ();
 			m_InHeader.write ((const char *)buf, len);
 			std::string line;
-			bool endOfHeader = false;
+			bool endOfHeader = false, connection = false;
 			while (!endOfHeader)
 			{
 				std::getline(m_InHeader, line);
@@ -349,9 +350,33 @@ namespace client
 					if (line == "\r") endOfHeader = true;
 					else
 					{
-						if (m_Host.length () > 0 && !line.compare(0, 5, "Host:"))
+						// strip up some headers
+						static const std::vector<std::string> excluded // list of excluded headers
+						{
+							"Keep-Alive:", "X-I2P" 
+						};
+						bool matched = false;
+						for (const auto& it: excluded)
+							if (boost::iequals (line.substr (0, it.length ()), it))
+							{
+								matched = true;
+								break;
+							}
+						if (matched) break;
+
+						// replace some headers
+						if (!m_Host.empty () && boost::iequals (line.substr (0, 5), "Host:"))
 							m_OutHeader << "Host: " << m_Host << "\r\n"; // override host
-						else
+						else if (boost::iequals (line.substr (0, 11), "Connection:"))
+						{
+							auto x = line.find("pgrade");
+							if (x != std::string::npos && x && std::tolower(line[x - 1]) != 'u') // upgrade or Upgrade
+								m_OutHeader << line << "\n";
+							else
+								m_OutHeader << "Connection: close\r\n";
+							connection = true;
+						}	
+						else // forward as is	
 							m_OutHeader << line << "\n";
 					}
 				}
@@ -361,6 +386,9 @@ namespace client
 
 			if (endOfHeader)
 			{
+				// add Connection if not presented
+				if (!connection)
+					m_OutHeader << "Connection: close\r\n";
 				// add X-I2P fields
 				if (m_From)
 				{
@@ -632,6 +660,12 @@ namespace client
 
 	void I2PServerTunnel::Stop ()
 	{
+		if (m_PortDestination)
+			m_PortDestination->ResetAcceptor ();
+		auto localDestination = GetLocalDestination ();
+		if (localDestination)
+			localDestination->StopAcceptingStreams ();
+		
 		ClearHandlers ();
 	}
 
@@ -1141,3 +1175,4 @@ namespace client
 	}
 }
 }
+
