@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2021, The PurpleI2P Project
+* Copyright (c) 2013-2022, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -1478,14 +1478,21 @@ namespace client
 						auto session = FindSession (sessionID);
 						if (session)
 						{
-							i2p::data::IdentityEx dest;
-							dest.FromBase64 (destination);
-							if (session->Type == eSAMSessionTypeDatagram)
-								session->GetLocalDestination ()->GetDatagramDestination ()->
-									SendDatagramTo ((uint8_t *)eol, payloadLen, dest.GetIdentHash ());
-							else // raw
-								session->GetLocalDestination ()->GetDatagramDestination ()->
-									SendRawDatagramTo ((uint8_t *)eol, payloadLen, dest.GetIdentHash ());
+							auto localDest = session->GetLocalDestination ();
+							auto datagramDest = localDest ? localDest->GetDatagramDestination () : nullptr;
+							if (datagramDest)
+							{	
+								i2p::data::IdentityEx dest;
+								dest.FromBase64 (destination);
+								if (session->Type == eSAMSessionTypeDatagram)
+									datagramDest->SendDatagramTo ((uint8_t *)eol, payloadLen, dest.GetIdentHash ());
+								else if (session->Type == eSAMSessionTypeRaw)
+									datagramDest->SendRawDatagramTo ((uint8_t *)eol, payloadLen, dest.GetIdentHash ());
+								else
+									LogPrint (eLogError, "SAM: Unexpected session type ", (int)session->Type, "for session ", sessionID);
+							}	
+							else
+								LogPrint (eLogError, "SAM: Datagram destination is not set for session ", sessionID);
 						}
 						else
 							LogPrint (eLogError, "SAM: Session ", sessionID, " not found");
