@@ -191,13 +191,13 @@ namespace proxy
 
 	void SOCKSHandler::AsyncSockRead()
 	{
-		LogPrint(eLogDebug, "SOCKS: async sock read");
+		LogPrint(eLogDebug, "SOCKS: Async sock read");
 		if (m_sock) {
 			m_sock->async_receive(boost::asio::buffer(m_sock_buff, socks_buffer_size),
 				std::bind(&SOCKSHandler::HandleSockRecv, shared_from_this(),
 				std::placeholders::_1, std::placeholders::_2));
 		} else {
-			LogPrint(eLogError,"SOCKS: no socket for read");
+			LogPrint(eLogError,"SOCKS: No socket for read");
 		}
 	}
 
@@ -206,19 +206,19 @@ namespace proxy
 		if (Kill()) return;
 		if (m_sock)
 		{
-			LogPrint(eLogDebug, "SOCKS: closing socket");
+			LogPrint(eLogDebug, "SOCKS: Closing socket");
 			m_sock->close();
 			m_sock = nullptr;
 		}
 		if (m_upstreamSock)
 		{
-			LogPrint(eLogDebug, "SOCKS: closing upstream socket");
+			LogPrint(eLogDebug, "SOCKS: Closing upstream socket");
 			m_upstreamSock->close();
 			m_upstreamSock = nullptr;
 		}
 		if (m_stream)
 		{
-			LogPrint(eLogDebug, "SOCKS: closing stream");
+			LogPrint(eLogDebug, "SOCKS: Closing stream");
 			m_stream.reset ();
 		}
 		Done(shared_from_this());
@@ -386,7 +386,7 @@ namespace proxy
 		if ( m_cmd != CMD_CONNECT )
 		{
 			// TODO: we need to support binds and other shit!
-			LogPrint(eLogError, "SOCKS: unsupported command: ", m_cmd);
+			LogPrint(eLogError, "SOCKS: Unsupported command: ", m_cmd);
 			SocksRequestFailed(SOCKS5_CMD_UNSUP);
 			return false;
 		}
@@ -399,7 +399,7 @@ namespace proxy
 					LogPrint(eLogError, "SOCKS: v5 unsupported address type: ", m_addrtype);
 				break;
 				case SOCKS4:
-					LogPrint(eLogError, "SOCKS: request with v4a rejected because it's actually SOCKS4");
+					LogPrint(eLogError, "SOCKS: Request with v4a rejected because it's actually SOCKS4");
 				break;
 			}
 			SocksRequestFailed(SOCKS5_ADDR_UNSUP);
@@ -426,7 +426,7 @@ namespace proxy
 							EnterState(GET5_AUTHNUM); //Initialize the parser at the right position
 						break;
 						default:
-							LogPrint(eLogError, "SOCKS: rejected invalid version: ", ((int)*sock_buff));
+							LogPrint(eLogError, "SOCKS: Rejected invalid version: ", ((int)*sock_buff));
 							Terminate();
 							return false;
 					}
@@ -456,7 +456,7 @@ namespace proxy
 							[[fallthrough]];
 #endif
 						default:
-							LogPrint(eLogError, "SOCKS: invalid command: ", ((int)*sock_buff));
+							LogPrint(eLogError, "SOCKS: Invalid command: ", ((int)*sock_buff));
 							SocksRequestFailed(SOCKS5_GEN_FAIL);
 							return false;
 					}
@@ -558,7 +558,7 @@ namespace proxy
 					if (m_parseleft == 0) EnterState(GET_PORT);
 				break;
 				default:
-					LogPrint(eLogError, "SOCKS: parse state?? ", m_state);
+					LogPrint(eLogError, "SOCKS: Parse state?? ", m_state);
 					Terminate();
 					return false;
 			}
@@ -576,10 +576,10 @@ namespace proxy
 
 	void SOCKSHandler::HandleSockRecv(const boost::system::error_code & ecode, std::size_t len)
 	{
-		LogPrint(eLogDebug, "SOCKS: received ", len, " bytes");
+		LogPrint(eLogDebug, "SOCKS: Received ", len, " bytes");
 		if(ecode)
 		{
-			LogPrint(eLogWarning, "SOCKS: recv got error: ", ecode);
+			LogPrint(eLogWarning, "SOCKS: Recv got error: ", ecode);
 			Terminate();
 			return;
 		}
@@ -589,7 +589,7 @@ namespace proxy
 			if (m_state == READY)
 			{
 				const std::string addr = m_address.dns.ToString();
-				LogPrint(eLogInfo, "SOCKS: requested ", addr, ":" , m_port);
+				LogPrint(eLogInfo, "SOCKS: Requested ", addr, ":" , m_port);
 				const size_t addrlen = addr.size();
 				// does it end with .i2p?
 				if ( addr.rfind(".i2p") == addrlen - 4) {
@@ -612,7 +612,7 @@ namespace proxy
 	void SOCKSHandler::SentSocksFailed(const boost::system::error_code & ecode)
 	{
 		if (ecode)
-			LogPrint (eLogError, "SOCKS: closing socket after sending failure because: ", ecode.message ());
+			LogPrint (eLogError, "SOCKS: Closing socket after sending failure because: ", ecode.message ());
 		Terminate();
 	}
 
@@ -621,7 +621,7 @@ namespace proxy
 		if (!ecode)
 		{
 			if (Kill()) return;
-			LogPrint (eLogInfo, "SOCKS: new I2PTunnel connection");
+			LogPrint (eLogInfo, "SOCKS: New I2PTunnel connection");
 			auto connection = std::make_shared<i2p::client::I2PTunnelConnection>(GetOwner(), m_sock, m_stream);
 			GetOwner()->AddHandler (connection);
 			connection->I2PConnect (m_remaining_data,m_remaining_data_len);
@@ -629,7 +629,7 @@ namespace proxy
 		}
 		else
 		{
-			LogPrint (eLogError, "SOCKS: closing socket after completion reply because: ", ecode.message ());
+			LogPrint (eLogError, "SOCKS: Closing socket after completion reply because: ", ecode.message ());
 			Terminate();
 		}
 	}
@@ -638,7 +638,7 @@ namespace proxy
 	{
 		if (ecode)
 		{
-			LogPrint (eLogError, "SOCKS: closing socket after sending reply because: ", ecode.message ());
+			LogPrint (eLogError, "SOCKS: Closing socket after sending reply because: ", ecode.message ());
 			Terminate();
 		}
 	}
@@ -652,14 +652,14 @@ namespace proxy
 		}
 		else
 		{
-			LogPrint (eLogError, "SOCKS: error when creating the stream, check the previous warnings for more info");
+			LogPrint (eLogError, "SOCKS: Error when creating the stream, check the previous warnings for more info");
 			SocksRequestFailed(SOCKS5_HOST_UNREACH);
 		}
 	}
 
 	void SOCKSHandler::ForwardSOCKS()
 	{
-		LogPrint(eLogInfo, "SOCKS: forwarding to upstream");
+		LogPrint(eLogInfo, "SOCKS: Forwarding to upstream");
 		EnterState(UPSTREAM_RESOLVE);
 		boost::asio::ip::tcp::resolver::query q(m_UpstreamProxyAddress, std::to_string(m_UpstreamProxyPort));
 		m_proxy_resolver.async_resolve(q, std::bind(&SOCKSHandler::HandleUpstreamResolved, shared_from_this(),
@@ -668,12 +668,12 @@ namespace proxy
 
 	void SOCKSHandler::AsyncUpstreamSockRead()
 	{
-		LogPrint(eLogDebug, "SOCKS: async upstream sock read");
+		LogPrint(eLogDebug, "SOCKS: Async upstream sock read");
 		if (m_upstreamSock) {
 			m_upstreamSock->async_read_some(boost::asio::buffer(m_upstream_response, SOCKS_UPSTREAM_SOCKS4A_REPLY_SIZE),
 				std::bind(&SOCKSHandler::HandleUpstreamSockRecv, shared_from_this(), std::placeholders::_1, std::placeholders::_2));
 		} else {
-			LogPrint(eLogError, "SOCKS: no upstream socket for read");
+			LogPrint(eLogError, "SOCKS: No upstream socket for read");
 			SocksRequestFailed(SOCKS5_GEN_FAIL);
 		}
 	}
@@ -685,7 +685,7 @@ namespace proxy
 				// we are trying to handshake but it failed
 				SocksRequestFailed(SOCKS5_NET_UNREACH);
 			} else {
-				LogPrint(eLogError, "SOCKS: bad state when reading from upstream: ", (int) m_state);
+				LogPrint(eLogError, "SOCKS: Bad state when reading from upstream: ", (int) m_state);
 			}
 			return;
 		}
@@ -694,7 +694,7 @@ namespace proxy
 
 	void SOCKSHandler::SocksUpstreamSuccess()
 	{
-		LogPrint(eLogInfo, "SOCKS: upstream success");
+		LogPrint(eLogInfo, "SOCKS: Upstream success");
 		boost::asio::const_buffers_1 response(nullptr, 0);
 		switch (m_socksv)
 		{
@@ -734,7 +734,7 @@ namespace proxy
 					SocksUpstreamSuccess();
 				} else {
 					// upstream failure
-					LogPrint(eLogError, "SOCKS: upstream proxy failure: ", (int) resp);
+					LogPrint(eLogError, "SOCKS: Upstream proxy failure: ", (int) resp);
 					// TODO: runtime error?
 					SocksRequestFailed(SOCKS5_GEN_FAIL);
 				}
@@ -744,30 +744,30 @@ namespace proxy
 			}
 		} else {
 			// invalid state
-			LogPrint(eLogError, "SOCKS: invalid state reading from upstream: ", (int) m_state);
+			LogPrint(eLogError, "SOCKS: Invalid state reading from upstream: ", (int) m_state);
 		}
 	}
 
 	void SOCKSHandler::SendUpstreamRequest()
 	{
-		LogPrint(eLogInfo, "SOCKS: negotiating with upstream proxy");
+		LogPrint(eLogInfo, "SOCKS: Negotiating with upstream proxy");
 		EnterState(UPSTREAM_HANDSHAKE);
 		if (m_upstreamSock) {
 			boost::asio::write(*m_upstreamSock, GenerateUpstreamRequest());
 			AsyncUpstreamSockRead();
 		} else {
-			LogPrint(eLogError, "SOCKS: no upstream socket to send handshake to");
+			LogPrint(eLogError, "SOCKS: No upstream socket to send handshake to");
 		}
 	}
 
 	void SOCKSHandler::HandleUpstreamConnected(const boost::system::error_code & ecode, boost::asio::ip::tcp::resolver::iterator itr)
 	{
 		if (ecode) {
-			LogPrint(eLogWarning, "SOCKS: could not connect to upstream proxy: ", ecode.message());
+			LogPrint(eLogWarning, "SOCKS: Could not connect to upstream proxy: ", ecode.message());
 			SocksRequestFailed(SOCKS5_NET_UNREACH);
 			return;
 		}
-		LogPrint(eLogInfo, "SOCKS: connected to upstream proxy");
+		LogPrint(eLogInfo, "SOCKS: Connected to upstream proxy");
 		SendUpstreamRequest();
 	}
 
@@ -775,11 +775,11 @@ namespace proxy
 	{
 		if (ecode) {
 			// error resolving
-			LogPrint(eLogWarning, "SOCKS: upstream proxy", m_UpstreamProxyAddress, " not resolved: ", ecode.message());
+			LogPrint(eLogWarning, "SOCKS: Upstream proxy", m_UpstreamProxyAddress, " not resolved: ", ecode.message());
 			SocksRequestFailed(SOCKS5_NET_UNREACH);
 			return;
 		}
-		LogPrint(eLogInfo, "SOCKS: upstream proxy resolved");
+		LogPrint(eLogInfo, "SOCKS: Upstream proxy resolved");
 		EnterState(UPSTREAM_CONNECT);
 		auto & service = GetOwner()->GetService();
 		m_upstreamSock = std::make_shared<boost::asio::ip::tcp::socket>(service);
